<?php

/**
 * ACLObject Class File
 *
 * @author zeroByte <dispyfree@googlemail.com>
 * @copyright mine 
 * @license LGPLv2
 */

/**
 * Object for managing Access objects (doesn't matter if they're control or request objects)
 */
class ACLObject extends CActiveRecord{
    
    
    /**
     * @var int The ID of the parent node in the tree. NULL indicates a root-node.
     */
    public $parent_id = NULL;
    
    /**
     * @var int The old parent-ID. This value is used to track parent-changes and enables to automagically update the underlying tree. 
     * @see parent_id
     */
    protected $old_parent_id = NULL;
    
    /**
     * @var int The left position (in the tree) of the node represented by the object. The positions is calculated using MPTT.
     */
    public $lft = NULL;
    
    /**
     * @var int The right position (in the tree) of the node represented by the object. The positions is calculated using MPTT.
     */
    public $rgt = NULL;
    
    
    /**
     * @var Tree    This value contains a tree object. Each ACLObject has got its own tree object, nevertheless they all access the same structures in the database. 
     */
    protected $tree = NULL;
    
    /**
     * @var string  The name of the table comprising the HABTM-relationship between Aros and Acos
     */
    public static $habtmTable = '{{aco_aro}}';
    
    /**
     * @var string  The name of the table containing the Aro-Rows
     */
    public static $aroTable   = '{{aro}}';
    
    /**
     * @var string  The name of the table containing the Aco-Rows
     */
    public static $acoTable  = '{{aco}}';
    
    /**
     * @var array   These are the columns reserved for internal use in the habtmTable. All other columns will automagically considered valid actions. [Currently not in use]
     * @see habtmTable
     */
    protected static $reservedColumns = array('id', 'aro_id', 'aco_id', 'aro_lft', 'aro_rgt', 'aco_lft', 'aco_rgt');
    
    
    /**
     * @var boolean Normally, objects of type ACLObject are assumed to have an underlying tree. This tree needs to be updated if the relation (e.g.: parent) is changed. Alas, PHP does _not_ provide multiple inheritance yet. Therewith objects behaving like ACLObjects which do not entail ensuing tree updates upon changes´can derived from ACLObject, this option disables the automatic tree update. 
     * @see tree
     */
    public $isTree = true;
    
    /**
     * 
     * @var boolean Enables or disables the strict-Behaviour: If strict-behaviour is disabled the requested objects will simply be created if they do not exist yet, otherwise an exception will be thrown
     */
    public $strict = false;
    
    /**
     *  Returns the static model of the specified AR class. 
     * @param string $className active record class name.
     * @return User the static model class
     */
    public static function model($className=__CLASS__)
    {
        return parent::model($className);
    }
    
    /**
     * This method is an equivalent of loadObjects
     * @see AclObject::loadObjects
     * @param mixed $identifier The identifier denoting the object
     * @param mixed $model the model to load the object from
     * @return mixed the object
     */
    protected function loadObject($identifier, $model = NULL){
        return $this->loadObjects($identifier, $model, true);
    }
    
    /**
     * This method is used to load Objects (either Aco or Aro) using convenient identifiers. 
     * In particular this is used to load Aro-/Aco-Objects associated to a collection
     * 
     * @param mixed $identifier The Identifier denoting the associated row in the ACL-system. 
     * Supported identifiers:
     * 1)   Array syntax: array('model' => 'MyModel', 'foreign_key' => myId)
     *          e.g.: model => User, foreign_key => the ID of the user (presumable AUTO_INCREMENT INT from the user row)
     * 2)   alias syntax: "MyAlias"
     *          e.g.: "Visitors", "Admins", "Authors"
     * 3)   direct syntax: pass your object derived from CActiveRecord directly
     *          e.g.: loadObject(User::model()->find(....))
     *          This will be automatically resolved to the first syntax. 
     *          Please be aware that the system does _not_ automatically create corresponding ACL-rows for the objects you pass in. 
     *          So if you pass a new object which has no corresponding aco/aro-row, this will lead to an exception.
     * 4)   Of course, you can pass the finished object directly. As many methods call this method without check, this is natural.
     * 
     * @param mixed $model   The model - this can be either an object of type CActiveRecord or a string which will be instantiated accordingly
     * @param boolean $onlyFirst    Determines whether to fetch only the first matching object, or all of them. 
     * @return type 
     */
    protected function loadObjects($identifier, $model = NULL, $onlyFirst = false){
        //There are several ways to define the object in question
        if(!$model)
            $model = $this;
        
        if(is_string($model))
            $model = new $model;
        
        if($onlyFirst)
                $method = 'find';
            else
                $method = 'findAll';
            
        //An alias is being used
        if(is_string($identifier)){
            
            $objects = $model->$method('alias=:alias', array('alias' => $identifier));
            
            if(!$objects){
                if($this->strict){
                    throw new Exception('Unknown alias for ACL-ObjectCollection');
                }
                else{
                    $obj = new $model;
                    $obj->alias   = $identifier;
                    $obj->save();
                    
                    $objects = 'find' ? $obj : array($obj);
               }
            }
            
        }
        
        //The object is searched by its model
        elseif(
                is_array($identifier) && 
                (isset($identifier['foreign_key']) && isset($identifier['model']))
            ){
                
            $objects = $model->$method('foreign_key = :foreign_key AND model = :model', array(':foreign_key' => $identifier['foreign_key'], ':model' => $identifier['model']));
            if(!$objects){
                if($this->strict){
                    throw new Exception('Unknown foreign key and/or model for ACL-ObjectCollection');
                }
                else{
                    $obj = new $model;
                    $obj->foreign_key   = $identifier['foreign_key'];
                    $obj->model         = $identifier['model'];
                    $obj->save();
                    
                    $objects = 'find' ? $obj : array($obj);
                }
                
            }
        }
        
        //The object is passed directly - do not do anything
        elseif(is_a($identifier, "ACLObjectCollection") || is_a($identifier, "ACLObject")){
            $objects = $identifier;
        }
        elseif(is_a($identifier, "CActiveRecord")){
            return $this->loadObjects( array('model' => get_class($identifier), 'foreign_key' => $identifier->id), $model, $onlyFirst);
        }
        else{
            throw new Exception('Unknown ACL-ObjectCollection specification');
        }
        
        return $objects;
    }
    
    /**
     * There are occasions where we must not call the regular, automatical save function. 
     * Especially the Tree-Object has to make direct changes to the lft- and rgt-values without invoking the automatic logic.
     * Disgarding this method will lead to infinite recursion loops. 
     * In fact, this method is only a wrapper to CActiveRecord::save
     * @see ACLObject::save
     * 
     * @param bool $runValidation   Whether to run the validation before saving the record
     * @param array $attributes additional parameters (see Yii-doc)
     * @return bool whether the save succeeded 
     */
    public function straightSave($runValidation = false, $attributes = NULL){
        return parent::save($runValidation, $attributes);
    }
    
    /**
     * This automatic save-method will automatically update the underlying tree if crucial values (e.g.: parent) are changed
     * @see CActiveRecord::save
     * @see ACLObject::straightSave
     * 
     * @param bool $runValidation   Whether to run the validation before saving the record
     * @param array $attributes additional parameters (see Yii-doc)
     * @return bool whether the save succeeded 
     */
    public function save($runValidation = false, $attributes = NULL){
       if($this->isTree){
           //If the item is new, we must add it to the tree
           if(!$this->id){
               return $this->id = $this->tree->addItem($this);
           }
           else{
               //Detect changes of the direct parent
               if( $this->old_parent_id != $this->parent_id){
                   return $this->tree->moveItem($this, $this->parent_id);
               }
           }
       }
      
       return parent::save($runValidation, $attributes);
    }
    
    /**
     * automatic: if the object is deleted, it's associated tree-rows need to be deleted too
     * @return bool whether the deletion succeeded
     */
    public function delete(){
        return $this->tree->deleteItem($this);
    }
    
    /**
     *  Checks whether the current object has got a parent
     * @return bool true if it has a parent - otherwise false
     */
    public function hasParent(){
        return ! $this->parent_id === NULL;
    }
    
    /**
     *  Returns the parent of the object
     * if the object has no parent or if it isn't existent, an exception is thrown
     * @return mixed    the parent object
     */
    public function getParent(){
        if(!$this->hasParent() || !$parent = $this->findByPk($this->parent_id))
            throw new Exception('This ACL-Object doesn\'t have a parent');
        return $parent;
    }
    
    /**
     * Returns all possible actions on the Acos
     * @see ACLObject::reservedColumns
     * @return array    all available actions
     */
    public static function getAllActions(){
        $columns = Yii::app()->db->getSchema()->getTable(self::$habtmTable)->getColumnNames();
        return array_diff($columns, self::$reservedColumns);
    }
    
    /**
     * Translates the given action-set to real actions.
     * @param mixed $actions The actions to be performed
     * Currently supported syntax:
     * 1)   array('action1', 'action2'...)
     * 2)   wildcard "*" will fetch all actions
     * @return array    the actions available on a specific object 
     */
    public function translateActions($actions){
        if(is_array($actions))
            return $actions;
        
        if($actions == '*'){
            return self::getAllActions();
        }
        
        //It's only one action
        
        return array($actions);
    }
    
    /**
     * automagic: saves the parent_id to track parent-changes
     * @see ACLObject::straightSave
     * @see ACLObject::parent_id
     * @see ACLObject::old_parent_id
     */
    public function afterFind(){
        $this->old_parent_id = $this->parent_id;
    }
    
}
?>
