<?php

/**
 * ACLObjectCollection Class File
 *
 * @author zeroByte <dispyfree@googlemail.com>
 * @copyright mine 
 * @license LGPLv2
 */

/**
 * This is the model class for ACL-collection tables
 *
 * The followings are the available columns in the table
 * @property integer $id    the unique ID of the row
 * @property string $alias  the (unique) alias of the row (can be empty)
 * @property string $model  the model the row relates to (can be empty)
 * @property integer $foreign_key   the foreign key of the related record of type $model (can be empty)
 */
class ACLObjectCollection extends ACLObject
{
    public $alias       = NULL;
    public $model        = NULL;
    public $foreign_key = NULL;
    
    public static $aroCollectionTable = '{{aro_collection}}';
    public static $acoCollectionTable = '{{aco_collection}}';
    
    /**
     * if the cache is activated (== Yii::app()->cache does exist) it's used to cache positional information by default
     * @var int determines the duration for the cached positions to expire
     */
    protected static $positionCacheExpiration = 3600;
    
    public $isTree = false;
    
    /**
     * Generates a unqiue string representation fo the given identifier and type
     * This is mainly used for caching purposes
     * @param mixed $identifier the identifier to represent
     * @param string $type the type to represent
     * @return string   the unique representation
     */
    public function getStringRepresentation($identifier, $type){
        $id = 'ACLObjCollection_'.$type;
        $spec = '';
        
        if(isset($identifier['alias'])){
            $spec .= 'ALIAS_'.$identifier['alias'];
        }
        else if(isset($identifier['model']) && isset($identifier['foreign_key'])){
            $spec .= 'RELATION_'.$identifier['model'].'_'.$identifier['foreign_key'];
        }
        else if(is_a($identifier, "ACLObjectCollection")){
            $spec .= 'BARE_'.$identifier->id;
        }
        else{
            throw new Exception('Invalid identifier');
        }
        
        //If the specific part precedes the preliminaries, the search will be much faster but nevertheless won't collide easily
        return $spec.$id;
    }
    
    /**
     * Each ACL-ObjectCollection _may_ (but doesn't need to) consist of several ACL-Objects. Fetch all of its lft/rgtg-combinations
     * @return  array       All of the combinations
     */
    public function fetchComprisedPositions(){
        
        $type = is_a($this, "AroCollection") ? 'Aro' : 'Aco';
        
        //First Check: Do we have caching facilities? ;)
        $id = $this->getStringRepresentation($this, $type);
        
        if(isset(Yii::app()->cache)){
            //Is this cached?
            $value = Yii::app()->cache->get($id);
            
            if($value)
                return unserialize($value);
        }
        
        //OK: we have to ask the database ^^
        $objects = $type::model()->findAll('collection_id = :collection_id', array(':collection_id' => $this->id));
        
        $positions = array();
        
        foreach($objects as $obj){
            $positions[] = array('lft' => $obj->lft, 'rgt' => $obj->rgt);
        }
        
        //Write it back into cache, if exists^^
        if(isset(Yii::app()->cache)){
            Yii::app()->cache->set($id, serialize($positions), self::$positionCacheExpiration);
        }
        
        return $positions;
    }
    
    /**
     * Checks whether the object associated to this collection is member of the given group
     * @param type $group 
     */
    public function is($group){
        //Get the group first 
        $type = is_a($this, "AroCollection") ? 'AroCollection' : 'AcoCollection';
        
        $group = $this->loadObject($group, $type);
       
        //If there's no such group, I certainly don't have any access :)
        if(!$group)
            return false;
        $positions = $this->fetchComprisedPositions();
        $aim_positions = $group->fetchComprisedPositions();
        
        foreach($aim_positions as $aim_position){
            foreach($positions as $position){
                if($aim_position['lft'] <= $position['lft'] && $aim_position['rgt'] >= $position['rgt'])
                    return true;
            }
        }
        
        return false;
    }
    
    /**
     * Joins the given group by setting the parent of one child to the given one
     * @param type $group 
     */
    public function join($group){
        
        $group = $this->loadObject($group);
        $type = is_a($this, "AroCollection") ? 'Aro' : 'Aco';
        
        
        try{
        
           $transaction = Yii::app()->db->beginTransaction();     
           
            //Let's search a suitable child
            $cmd = Yii::app()->db->createCommand()
                       ->select('id')
                    ->from('{{'.$type.'}}')
                    ->where(
                            array(
                                'and',
                                'parent_id IS NULL',
                                'collection_id = :id'
                            )
                            )
                    ->limit(1);
            $cmd->bindValue(':id', $this->id);

            $id = $cmd->queryScalar();


            //We only need a random object of the other collection
            $obj = $type::model()->find('collection_id = :id', array(':id' => $group->id));
                
            //If we don't have a child node without a parent, we have to create a new one
            if(!$id){
                $aclObject = new $type;
                $aclObject->collection_id = $this->id;
                $aclObject->parent_id = $obj->id;
                $aclObject->save();
            }
            else{
                $aclObject = $type::model()->findByPk($id);
                $aclObject->parent_id = $obj->id;
                $aclObject->save();
            }
            
            $transaction->commit();
        }
        catch(Exception $e){
            $transaction->rollback();
            throw new Exception('Unable to join group '.$group->id.' : '.$e->getMessage());
        }
        
        return true;
    }
    
    /**
     * Leaves the given group
     * @Todo    
     * @param type $group 
     */
    public function leave($group){
        $group = $this->loadObject($group);
        $type = is_a($this, "AroCollection") ? 'Aro' : 'Aco';
        
        $transaction = Yii::app()->db->beginTransaction();
        
        try{
            
            $objects = $type::model()->findAll(
                    'parent_id = :parent_id AND collection_id = :id',
                    array(
                        ':parent_id' => $group->id,
                        ':id'        => $this->id
                    )
               );
        
            foreach($objects as $object){
                if($object->parent_id == $group->id){
                    $object->parent_id = NULL;
                    $object->save();
                }
            }
            
            $transaction->commit();
        }
        catch(Exception $e){
            $transaction->rollback();
            throw new Exception("Unable to leave group ".$group->id.": ".$e->getMessage());
        }
        
        return true;
    }
    
    /**
     * This method takes care that every subordinate ACL-objects are properly removed 
     */
    public function beforeDelete(){
        parent::beforeDelete();
        
        //OK: we have to ask the database ^^
        $type = is_a($this, "AroCollection") ? 'Aro' : 'Aco';
        $objects = $type::model()->findAll('collection_id = :collection_id', array(':collection_id' => $this->id));
        
        foreach($objects as $object){
            if(!$object->delete())
                throw new RuntimeException('Unable to delete ACL-node of ACLCollection '.$this->id);
        }
        
        return true;
    }
    
    
    /**
     * Returns the static model of the specified AR class.
     * @param string $className active record class name.
     * @return AcoCollection the static model class
     */
    public static function model($className=__CLASS__)
    {
        return parent::model($className);
    }


    /**
     * @return array validation rules for model attributes.
     */
    public function rules()
    {
        // NOTE: you should only define rules for those attributes that
        // will receive user inputs.
        return array(
        );
    }

    /**
     * @return array customized attribute labels (name=>label)
     */
    public function attributeLabels()
    {
        return array(
            'id' => 'ID',
            'alias' => 'Alias',
            'model' => 'Model',
            'foreign_key' => 'Foreign Key',
        );
    }
} 