<?php

/**
 * AroCollection Class File
 *
 * @author zeroByte <dispyfree@googlemail.com>
 * @copyright mine 
 * @license LGPLv2
*/

/**
 * AroCollections represent bunches of Aro-objects associcated to one parent
 * This is the class you will use in your daily work
 */
class AroCollection extends ACLObjectCollection
{
    
    /**
     * We have to overwrite the table as we don't want to operate on a tree of collections but on a tree of nodes
     */
    public function init() {
        $this->tree = new Tree(self::$aroTable, 'aro');
	parent::init();	
        
        $behavior = new CompositeBehavior('Aro','collection_id', array('parent_id' => 'parent_id', 'id' => 'collection_id'));
        $this->attachBehavior('composite', $behavior);
    }
    
    /**
     * Looks up if the user is granted a specific action to the given object
     * @param   string|array    $obj    The object to be checked   
     * @param   string          $action the action to be performed
     * @return bool true if access is granted, false otherwise
     */
    public function may($obj, $action){
        
        //Create the crucial query ^^
        $cmd = Yii::app()->db->createCommand()
                ->select('COUNT(*)')
                ->from(self::$habtmTable.' map');
        $cmd = $this->addPermissionCheck($cmd, $obj, $action);
        $cmd = $cmd->limit(1);
        
        $result = $cmd->queryScalar();

        return $result != false;
    }
    
    /**
     * Grants the object denoted by the $obj-identifier the given actions
     * @param type $obj the object identifier
     * @param array $actions the actions to grant
     * @return bool 
     */
    public function grant($obj, $actions){
        $obj = $this->loadObject($obj, "AcoCollection");
        $aro = Aro::model()->find('collection_id = :id', array(':id' => $this->id));
        $aco = Aco::model()->find('collection_id = :id', array(':id' => $obj->id));
        if(!$aro->grant($aco, $actions)){
           // throw new Exception('Unable to grant permissions on node'.$aro->id);
        }
        return true;
    }
    
    /**
     * Denies the object denoted by the $obj-identifier the given actions
     * @param type $obj the object identifier
     * @param array $actions the actions to deny
     * @return bool 
     */
    public function deny($obj, $actions){
        //if we grant someone rights, it's sufficient to do that with one AR-node. But now we need to traverse over all of them :X
        $obj = $this->loadObject($obj, "AcoCollection");
        $aros = Aro::model()->findAll('collection_id = :id', array(':id' => $this->id));
        $aco = Aco::model()->find('collection_id = :id', array(':id' => $obj->id));
        foreach($aros as $aro){
            //We don't need to check this: not every node has to have access. Therefore not every node can be denied access 
            $aro->deny($aco, $actions);
        }
        
        return true;
    }
    
    /**
     * Adds the (complicated) permission check to the given query
     * @param   CDbCommand  $cmd    the DB-command to add the check to
     * @param   unknown     $obj    An Object identifier 
     * @return  CDbCommand  The extended Db-command
     */
    protected function addPermissionCheck(&$cmd, $obj, $action){
        
        //Fetch all the positions
        $aroPositions = $this->fetchComprisedPositions();
        
        $acoCollection = new AcoCollection;
        $acoCollection = $acoCollection->loadObject($obj);
        
        if(!is_a($acoCollection, 'AcoCollection'))
            throw new Exception("Invalid Aco-Collection");
        
        $acoPositions = $acoCollection->fetchComprisedPositions();

        $conditions = array(
                'and',
                    array(
                        'and',
                    $this->addPositionCheck($aroPositions, 'aro'),
                    $this->addPositionCheck($acoPositions, 'aco')
                    ),
                'map.'.$action.' IS True'
            );

        $cmd->setWhere($conditions);
        
        return $cmd;
    }
    
    /**
     * Builds a single SQL-statement comprising all given positions
     * This SQL-statement will match all those rows being located somewhere left from lft and right from rgt of each given position
     * @param array $positions All positions to include in our statement
     * @param string $type aco/aro
     * @return string the finished SQL-statement
     */
    public function addPositionCheck($positions, $type){
        $new = $positions[0];
        $expression = 'map.'.$type.'_lft <= '.$new['lft'].' && map.'.$type.'_rgt >= '.$new['rgt'];
        
        //If we're at the end of our recursion.. nothing left to do ;)
        if(sizeof($positions) <= 1)
            return $expression;
        
        //there are more recursion steps to take
        $preparedConditions = array(
            'or',
            $expression,
            $this->addPositionCheck(array_slice($positions, 1), $type)
        );
        
        return $preparedConditions;
    }
    
    
    public function tableName(){
        return self::$aroCollectionTable;
    }
    
    /**
     * @return array relational rules.
     */
    public function relations()
    {
        // NOTE: you may need to adjust the relation name and the related
        // class name for the relations automatically generated below.
        return array(
            array(self::HAS_MANY, 'Aro', 'id')
        );
    }
    
    
    /**
     * Returns the static model of the specified AR class.
     * @param string $className active record class name.
     * @return AcoCollection the static model class
     */
    public static function model($className=__CLASS__)
    {
        return parent::model($className);
    }
} 