<?php

/**
 * RestrictedActiveRecord Class File
 *
 * @author zeroByte <dispyfree@googlemail.com>
 * @copyright mine 
 * @license LGPLv2
 */

/**
 * This class is intended as a base for objects which have restrictions on their access
 * It automatically checks, if the current user has the permissions to commit the regular CRUD-tasks
 */
class RestrictedActiveRecord extends CActiveRecord{
    
    /**
     * This is used to temporarily disable the Access  Check
     * This may be needed if for example the user is a guest and has therefore no update-rights (he doesn't have an ARO representation) on his own objects
     * @var boolean
     */
    public $byPassCheck = false;
    
    /**
     * This variable can be used to access the resource in attendance of another ARO-collection than the current user
     * default is to NULL which means that the user denoted by Yii::app()->user is used for access checking
     * @var mixed
     */
    public $inAttendance = NULL;
    
    /**
     * @var string  The model to use for automatic access checing (as Aro-Collection). Default: "User" 
     */
    public $model = 'User';
    
    /**
     * The following functions generates the CDbCriteria necessary to filter all accessable rows
     * The CDbCriteria is solely passsed to the wrapped methods
     * @param sql $conditions the conditions being passed to the real method
     * @param array $params the params being passed to the real method
     * @return CDbCriteria the criteria assuring that the user only gets what he has access to
     */
    protected function generateAccessCheck($conditions = '', $params = array()){
        if(!is_string($conditions) && get_class($conditions) == 'CDbCriteria'){
            $criteria = $conditions;
        }
        else{
            $criteria = new CDbCriteria;
            $criteria->condition = $conditions;
            $criteria->params = $params;
        }
        
        //If the check is bypassed, return criteria without check
        if($this->byPassCheck)
            return $criteria;
        
        $criteria->distinct = true; //Important: there can be multiple locations which grant permission
        
            //Inner join to get the collection associated with this content
            $collection = 'INNER JOIN `'.ACLObjectCollection::$acoCollectionTable.'` AS acoC ON acoC.model = :RAR_model AND acoC.foreign_key = t.id';
                $criteria->params[':RAR_model'] = get_class($this);
            
            //Inner join to the associated aco-nodes themselves to get the positions
            $nodes = ' INNER JOIN `'.ACLObjectCollection::$acoTable.'` AS aco ON aco.collection_id = acoC.id';
            
            //Last join: the connecting table between Aro and Aco
            //But before: fetch the positions of the current user
            $aroCollection = AroCollection::model()->find('model = :model AND foreign_key = :foreign_key', 
                    array(':model'=> $this->model, ':foreign_key' => Yii::app()->user->getId()));
            
            //If we are nobody... we are a guest^^
            if(!$aroCollection){
                 $aroCollection = AroCollection::model()->find('alias = :alias', 
                    array(':alias' => 'Guest'));
                 
                 //If there's no guest group... we are nobody and we may nothing ;)
                 if(!$aroCollection)
                     return array();
            }
               
            
            $positions = $aroCollection->fetchComprisedPositions();
            $positionCheck = $aroCollection->addPositionCheck($positions, "aro");
            
            //Now, join connecting table
            $connection = ' INNER JOIN `'.ACLObject::$habtmTable.'` AS map ON map.aco_id = aco.id AND '.$positionCheck.' AND map.read IS TRUE';
            
       $joins = $collection.$nodes.$connection;
       
       $criteria->join .= $joins;
       
       return $criteria;
    }
    
    
    public function find($conditions = '', $params = array()){
       return parent::find($this->generateAccessCheck($conditions, $params));
    }
    
    public function findByAttributes($attributes, $conditions = '', $params = array()){
       return parent::findByAttributes($attributes, $this->generateAccessCheck($conditions, $params));
    }
    
    public function findByPk($pk, $conditions = '', $params = array()){
       return parent::findByPk($pk, $this->generateAccessCheck($conditions, $params));
    }
    
    public function findBySQL($sql, $params = array()){
       return parent::find($this->generateAccessCheck($sql, $params));
    }
    
    
    public function findAll($conditions = '', $params = array()){
       return parent::findAll($this->generateAccessCheck($conditions, $params));
    }
    
    public function findAllByAttributes($attributes, $conditions = '', $params = array()){
       return parent::findAllByAttributes($attributes, $this->generateAccessCheck($conditions, $params));
    }
    
    public function findAllByPk($pk, $conditions = '', $params = array()){
       return parent::findAllByPk($pk, $this->generateAccessCheck($conditions, $params));
    }
    
    public function findAllBySQL($sql, $params = array()){
       return parent::findAll($this->generateAccessCheck($sql, $params));
    }
    
    /**
     * This method checks whether the user has the right to update the current record
     * By default, it's always allowed to create a new object. This object is automatically assigned to the user who created it with full permissions
     */
    public function beforeSave(){
        parent::beforeSave();
        //The Record is updated
        if(!$this->isNewRecord){
            $aro = $this->getUser();
            
            if(!$aro->may($this, 'update'))
                throw new RuntimeException('You are not allowed to update this record');            
        }
        
        return true;
    }
    
    /**
     * This method checks whether the user has the right to delete the current record
     * 
     */
    public function beforeDelete(){
        parent::beforeDelete();
        
        $aro = $this->getUser();
        if(!$aro->may($this, 'delete'))
                throw new RuntimeException('You are not allowed to delete this record');
        
        //Ok he has the right to do that - remove all the ACL-objects associated with this object
        $aco = AcoCollection::model()->find('model = :model AND foreign_key = :key', array(':model' => get_class($this), ':key' => $this->id));
        
        if(!$aco)
            throw new RuntimeException('No associated Aco-Collection!');
        $aco->delete();
        
        return true;
    }
    
    
    /**
     * This method takes care to assign individual rights to newly created objects
     * 
     * @param CEvent $evt 
     */
    public function afterSave(){
        parent::afterSave();
        if($this->isNewRecord){
            $aro = $this->getUser();
            //As the object is newly created, it needs a representation
            //If strict mode is disabled, this is not necessary
            $aco = new AcoCollection();
            $aco->model = get_class($this);
            $aco->foreign_key = $this->getPrimaryKey();
            
            if(!$aco->save()){
                throw new RuntimeException('Unable to create corresponding Aco-Collection for new '.get_class($this));
            }
            
            $aro->grant($aco, '*');
        }
    }
    
    
    protected function getUser(){
        
        if($this->inAttendance != NULL)
            return $this->inAttendance;
        
        $user = Yii::app()->user;
        $aro = AroCollection::model()->find('model = :model AND foreign_key = :foreign_key', 
                array('model' => $this->model, 'foreign_key' => $user->id));
        if(!$aro)
            throw new RuntimeException('Invalid Aro-Collection');
        return $aro;
    }
    
    
}
?>
